function [varargout]=gmdouble(varargin)
%% gmdouble replicates internal boundary faces and mesh nodes on the boundary
%%   Matlab: [brep2 {, mesh2}] = ...
%%             gmdouble(brep, faceindexlist {,mesh});
%%   Tcl/Tk: gmset {brep2 {mesh2}} \
%%                 [gmdouble $brep $faceindexlist { $mesh}]
%% This routine doubles internal boundaries, i.e., replaces them with two
%% coincident child faces. This routine takes a brep whose regions have
%% internal boundaries.  Some subset of those internal boundaries is
%% specified by faceindexlist; this is a list of strings specifying the
%% names of the faces of dimension d-1 that are internal boundaries to be
%% doubled.  If * is specified ({'*'} in matlab), then all internal
%% boundaries are doubled.  The routine replaces all the requested
%% internal boundaries with two copies.  It may also duplicate lower
%% dimensional faces that interconnect the specified internal boundaries.
%% The resulting brep is the return argument brep2.  The optional mesh
%% argument to this routine is a mesh generated for the original brep by
%% gmmeshgen.  The gmdouble routine duplicates nodes that sit on faces
%% that were duplicated.  Then for each simplex it determines which of
%% the duplicated nodes go with that simplex.  The resulting mesh is the
%% return argument mesh2. The simplices in mesh2 follow the same
%% numbering to simplices in mesh.  The vertices in mesh2 have the same
%% global numbers as in mesh, except duplicates of vertex i are renumbered
%% i+k*(n+1), where k is the copy number and n is the highest global number
%% occurring in the original mesh.  The global tolerance setting is used 
%% by this routine.
gm_errmsg('gmdouble')
