function bdryval = gm_bdrycond(proc, userdata, points)
%  bdryval = gm_bdrycond(proc, userdata, points)
% This routine computes boundary conditions.  proc is a string
% that defines the function (see the documentation).  points (a zba)
% is the points to evaluate the function at.  This routine
% is usually invoked by gmfem.
% Returns a zba

proc = gm_strim(proc);
if proc(1) ~= '('
  error('boundary-cond procedure must start with a parenthesis')
end
if proc(length(proc)) ~= ')'
  error('Unbalanced parentheses in boundary-cond procedure');
end

proc(1) = ' ';
proc(length(proc)) = ' ';
proc = gm_strim(proc);
[first, rest] = strtok(proc);
first = gm_strim(first);
rest = gm_strim(rest);
[n,di] = size(points);
if strcmp(first, 'const')
  bdryval = zba(str2num(rest) * ones(n,1));
elseif strcmp(first, 'vecformula') 
  formula = gm_percentsub(rest, 'double(points(:,', '))');
  bdryval = zba(eval(formula));
elseif strcmp(first, 'formula')
  formula = gm_percentsub(rest, 'double(points(i,', '))');
  bdryval = zba(-ones(n,1));
  for i = 1 : n
    bdryval(i) = eval(formula);
  end
else
  error('Unknown boundary-condition keyword');
end



% ------------------------------------------------------------------
% Copyright (c) 1998 by Cornell University.  All rights reserved.
% See the accompanying file 'Copyright' for authorship information,
% the terms of the license governing this software, and disclaimers
% concerning this software.
% ------------------------------------------------------------------
% This file is part of the QMG software.  
% Version 2.0 of QMG, release date RELDATE
% ------------------------------------------------------------------
