// ------------------------------------------------------------------
// qnamesp.h
//
// This file contains forward declarations of most of the classes
// in namespace QMG.
// ------------------------------------------------------------------
// Author: Stephen A. Vavasis
// Copyright (c) 1999 by Cornell University.  All rights reserved.
// 
// See the accompanying file 'Copyright' for authorship information,
// the terms of the license governing this software, and disclaimers
// concerning this software.
// ------------------------------------------------------------------
// This file is part of the QMG software.  
// Version 2.0 of QMG, release date September 3, 1999.
// ------------------------------------------------------------------





#ifndef QMGNAMESP_H
#define QMGNAMESP_H

#ifdef _MSC_VER
#if _MSC_VER == 1200
#pragma warning (disable: 4786) 
#pragma warning (disable: 4788)
#endif
#endif



// STL headers used by QMG


#include <string>
#include <iostream>
#include <memory>
#include <vector>
#include <map>
#include <queue>
#include <iterator>
#ifdef USE_SSTREAM
#  include <sstream>
#else
#  include <stringstream>
#endif
#include <algorithm>
#include <new>
#include <iomanip>
#include <cmath>
#include <complex>
#include <fstream>
#include <ctime>


#ifdef _MSC_VER
#pragma warning (disable: 4786) 
#pragma warning (disable: 4788)
#endif


namespace QMG {
  using std::string;
  using std::ostream;
  using std::istream;
  using std::vector;
  using std::map;
  using std::multimap;
  using std::priority_queue;
  using std::pair;
  using std::sort;
  using std::istringstream;
  using std::ostringstream;
#if MATH_IN_STD_NAMESPACE
  using std::fabs;
  using std::sqrt;
  using std::atan2;
  using std::sin;
  using std::cos;
  using std::pow;
  using std::toupper;
  using std::isspace;
#endif


  template <class T, class U, class V> 
    class Triple {
    public:
      T first;
      U second;
      V third;
      Triple(): first(T()), second(U()), third(V()) { }
      Triple(const T& t, const U& u, const V& v) : first(t), second(u), third(v) { }
  };
  
  
  template<class T, int sz>
    class StatVector;

  class Matrix;
  class IntMatrix;
  class Brep;
  class Brep_From_FrontEnd;
  class Brep_Under_Construction;
  class SimpComplex;
  class SimpComplex_From_FrontEnd;
  class SimpComplex_Under_Construction;
  class Error;
  class Error_Message;
  class Error_Task;
  class PatchMath;
  class Point;
  class UnionFind;
  typedef unsigned int UInt32; 
  typedef double Real;

  const int MAXDIM = 3;
  const Real BIG_REAL = 1e308;
  const Real MACHINE_EPS = 2.220446049250313e-16;
  const Real PI = 3.14159265358979;
  extern int compare_nocase(const string& s1, const string& s2);
  extern Real random_real(int init = 0);
  extern void throw_error(const string& s);
  enum Object_Type_Code {BREP, SIMPCOMPLEX, UNKNOWN};
  enum PatchType {BEZIER_TRIANGLE, BEZIER_QUAD};
}


#endif
