// ------------------------------------------------------------------
// qmg_gui.h
//
// This file contains the definitions of the class that makes the
// panel during mesh generation.
// ------------------------------------------------------------------
// Author: Stephen A. Vavasis
// Copyright (c) 1999 by Cornell University.  All rights reserved.
// 
// See the accompanying file 'Copyright' for authorship information,
// the terms of the license governing this software, and disclaimers
// concerning this software.
// ------------------------------------------------------------------
// This file is part of the QMG software.  
// Version 2.0 of QMG, release date September 3, 1999.
// ------------------------------------------------------------------


#ifndef QMG_GUI_H
#define QMG_GUI_H


#include "qmgnamesp.h"
#include "qbrep.h"
#include "qlog.h"
#include "qsimpcomp.h"

extern "C" {
#include <time.h>
}

namespace QMG {
  namespace FrontEnd {
    class Interpreter;
  }
}



class QMG::MG::Meshgen_gui {
private:

  // Data lives here
  bool active_;
  QMG::FrontEnd::Interpreter& interp_;
  Logstream& logstr_;
  time_t last_update_time_;
  const SimpComplex* scp_;
  const BoxFaceDag* dagp_;

  // No copying, no assignment.
  Meshgen_gui(const Meshgen_gui& o) : interp_(o.interp_), logstr_(o.logstr_) { }
  void operator=(const Meshgen_gui&) { }

public:
  Meshgen_gui(bool active, 
    QMG::FrontEnd::Interpreter& interp, 
    const Brep& brep,
    Logstream& logstr);
  void associate_sc_and_dag(const SimpComplex& sc, const BoxFaceDag& dag);
  void check_update();
  void update();
  void operation(const string& s);
  void operation2(const string& s);
  void indicate_error_has_occurred(const string& errmsg);

  ~Meshgen_gui();
};



#endif
