// ------------------------------------------------------------------
// gmmeshsize.cpp
//
// This file contains a routine to measure the size
// of a mesh (number of nodes and elements).
// ------------------------------------------------------------------
// Author: Stephen A. Vavasis
// Copyright (c) 1999 by Cornell University.  All rights reserved.
// 
// See the accompanying file 'Copyright' for authorship information,
// the terms of the license governing this software, and disclaimers
// concerning this software.
// ------------------------------------------------------------------
// This file is part of the QMG software.  
// Version 2.0 of QMG, release date September 3, 1999.
// ------------------------------------------------------------------

#include "qsimpcomp.h"
#include "qfrontend.h"

#define GM_ROUTINE_NAME gmmeshsize
#define GM_ROUTINE_NAME_Q "gmmeshsize"


namespace {
  static void worker(const QMG::FrontEnd::ArgValType& argvalhandle,
    QMG::FrontEnd::ReturnValType& returnvalhandle,
    QMG::FrontEnd::Interpreter& interp) {
    
    using namespace QMG;
    using namespace QMG::FrontEnd;
    
    argvalhandle.verify_nargin(1, 1, GM_ROUTINE_NAME_Q);
    returnvalhandle.verify_nargout(2, 2, GM_ROUTINE_NAME_Q);
    SimpComplex_From_FrontEnd sc = argvalhandle.get_simpcomp(0);

    int nn = sc.num_nodes();
    int ne = sc.num_elements();
    returnvalhandle.put_int(0,nn);
    returnvalhandle.put_int(1,ne);
  }
}
 
#include "gm_entrypoint.cpp"


