// ------------------------------------------------------------------
// qbrep_constr.cpp
//
// This file contains member functions for qbrep_constr.h
// ------------------------------------------------------------------
// Author: Stephen A. Vavasis
// Copyright (c) 1999 by Cornell University.  All rights reserved.
// 
// See the accompanying file 'Copyright' for authorship information,
// the terms of the license governing this software, and disclaimers
// concerning this software.
// ------------------------------------------------------------------
// This file is part of the QMG software.  
// Version 2.0 of QMG, release date September 3, 1999.
// ------------------------------------------------------------------



#include "qbrep_constr.h"


// See qbrep_constr.h for comments.  This code is a mess because
// of the large number of subcases.  But it's not sophisticated.


extern "C" {
#include <stdlib.h>
#include <stdio.h>
}

#ifdef USE_DEBUG_MALLOC

#define malloc(x) debug_malloc(x,__LINE__,__FILE__)
#define free(x) debug_free(x,__LINE__,__FILE__)

extern "C" {
  void* debug_malloc(size_t sz, int lineno, const char* filename);
  void debug_free(void* ptr, int lineno, const char* filename);
}

#endif


namespace {
  using namespace QMG;
  void* malloc1(size_t sz) {
    char tmp[100];
    void* p = malloc(sz);
    if (p == 0) {
      sprintf(tmp, "Out of memory on malloc of size %d", sz);
      throw_error(tmp);
    }
    return p;
  }
}


QMG::Brep_Under_Construction::
Brep_Under_Construction(int intrinsic_dim, int embedded_dim) :
Brep(static_cast<Cptc_GeomDesc*>(malloc1(sizeof(Cptc_GeomDesc))))  {
  geo_ -> intrinsic_dimension = intrinsic_dim;
  geo_ -> embedded_dimension = embedded_dim;
  geo_ -> num_properties = 0;
  geo_ -> num_control_pts = 0;
  geo_ -> num_top_vertices = 0;
  geo_ -> num_top_edges = 0;
  geo_ -> num_top_surfaces = 0;
  geo_ -> num_top_regions = 0;
  geo_ -> control_points = 0;
  geo_ -> top_vertices = 0;
  geo_ -> top_edges = 0;
  geo_ -> top_surfaces = 0;
  geo_ -> top_regions = 0;
  if (embedded_dim < 2 || embedded_dim > 3) {
    free(geo_);
    throw_error("Embedded dimension out of range");
  }
  if (intrinsic_dim < 0 || intrinsic_dim > embedded_dim) {
    free(geo_);
    throw_error("Intrinsic dimension out of range");
  }
}

QMG::Brep_Under_Construction::Propval_inserter::
Propval_inserter(Brep_Under_Construction& brep) : brep_(brep) {
  geo_ = brep.geo_;
  if (geo_-> num_properties != 0) 
    throw_error("Illegal to create Control_pointer_inserter more than once");
  maxsize_ = 1;
  geo_ -> props = 
    static_cast<char**>(malloc1(sizeof(char*)));
  geo_ -> values = 
    static_cast<char**>(malloc1(sizeof(char*)));
}

int 
QMG::Brep_Under_Construction::Propval_inserter::
insert_propval(const string& prop, const string& val) {
  int insertpos = geo_ -> num_properties;
  if (insertpos == maxsize_) {
    int newsize = maxsize_ * 2 + 1;
    char** new_props = 
      static_cast<char**>(malloc1(sizeof(char*) * newsize));
    char** new_vals = 
      static_cast<char**>(malloc1(sizeof(char*) * newsize));
    for (int j = 0; j < insertpos; ++j) {
      new_props[j] = geo_ -> props[j];
      new_vals[j] = geo_ -> values[j];
    }
    free(geo_ -> props);
    free(geo_ -> values);
    maxsize_ = newsize;
    geo_ -> props = new_props;
    geo_ -> values = new_vals;
  }
  int l1 = prop.length();
  geo_ -> props[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l1 + 1)));
    prop.copy(geo_ -> props[insertpos], l1);
    geo_ -> props[insertpos][l1] = 0;
  int l2 = val.length();
  geo_ -> values[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l2 + 1)));
    val.copy(geo_ -> values[insertpos], l2);
    geo_ -> values[insertpos][l2] = 0;
  ++(geo_ -> num_properties);
  return insertpos;
}


QMG::Brep_Under_Construction::Control_point_inserter::
Control_point_inserter(Brep_Under_Construction& brep) : brep_(brep) {
  geo_ = brep.geo_;
  if (geo_-> control_points != 0) 
    throw_error("Illegal to create Control_pointer_inserter more than once");
  maxsize_ = 1;
  geo_-> control_points = 
    static_cast<Cptc_GPoint*>(malloc1(sizeof(Cptc_GPoint)));
  geo_ -> num_control_pts = 0;
}

int 
QMG::Brep_Under_Construction::Control_point_inserter::
insert_control_point(const vector<Real>& coord) {
  int insertpos = geo_ -> num_control_pts;
  if (geo_ -> embedded_dimension != coord.size())
    throw_error("Control point wrong length");
  if (insertpos == maxsize_) {
    int newsize = maxsize_ * 2 + 1;
    Cptc_GPoint* new_control_points = 
      static_cast<Cptc_GPoint*>(malloc1(sizeof(Cptc_GPoint) * newsize));
    for (int j = 0; j < insertpos; ++j) {
      new_control_points[j] = geo_ -> control_points[j];
    }
    free(geo_ -> control_points);
    maxsize_ = newsize;
    geo_ -> control_points = new_control_points;
  }
  for (int ii = 0; ii < coord.size(); ++ii) {
    geo_ -> control_points[insertpos].coord[ii] = coord[ii];
  }
  ++(geo_ -> num_control_pts);
  return insertpos;
}

QMG::Brep_Under_Construction::Top_face_inserter::
Top_face_inserter(Brep_Under_Construction& brep, int dim) : brep_(brep) {
  geo_ = brep.geo_;
  dim_ = dim;
  maxsize_ = 1;
  if (dim_ < 0 || dim_ > geo_ -> intrinsic_dimension)
    throw_error("Dimension out of range");
  if (dim_ == 0) {
    if (geo_-> top_vertices != 0) 
      throw_error("Illegal to create top vertex inserter more than once");
    geo_-> top_vertices = 
      static_cast<Cptc_TVertex*>(malloc1(sizeof(Cptc_TVertex)));
    geo_ -> num_top_vertices = 0;
  }
  else if (dim_ == 1) {
    if (geo_ -> top_edges != 0) 
      throw_error("Illegal to create top edge inserter more than once");
    geo_ -> top_edges = 
      static_cast<Cptc_TEdge*>(malloc1(sizeof(Cptc_TEdge)));
    geo_ -> num_top_edges = 0;
  }
  else if (dim_ == 2) {
    if (geo_ -> top_surfaces != 0)
      throw_error("Illegal to create top surface inserter more than once"); 
    geo_ -> top_surfaces = 
      static_cast<Cptc_TSurface*>(malloc1(sizeof(Cptc_TSurface)));
    geo_ -> num_top_surfaces = 0;
  }
  else if (dim_ == 3) {
    if (geo_ -> top_regions != 0) 
      throw_error("Illegal to create top region inserter more than once");
    geo_ -> top_regions = 
      static_cast<Cptc_TRegion*>(malloc1(sizeof(Cptc_TRegion)));
    geo_ -> num_top_regions = 0;
  }
  else {
    throw_error("Unknown dimension");
  }
}


QMG::Brep::Face_Spec 
QMG::Brep_Under_Construction::Top_face_inserter::
insert_top_face(const string& name) {
  if (dim_ == 0) {
    int insertpos = geo_ -> num_top_vertices;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      Cptc_TVertex* new_vertices = 
        static_cast<Cptc_TVertex*>(malloc1(sizeof(Cptc_TVertex) * newsize));
      for (int j = 0; j < insertpos; ++j) {
        new_vertices[j] = geo_ -> top_vertices[j];
      }
      free(geo_ -> top_vertices);
      maxsize_ = newsize;
      geo_ -> top_vertices = new_vertices;
    }      
    geo_ -> top_vertices[insertpos].props = 0;
    geo_ -> top_vertices[insertpos].values = 0;
    geo_ -> top_vertices[insertpos].num_properties = 0;
    geo_ -> top_vertices[insertpos].control_pnt_indx = -1;
    int l = name.length();
    geo_ -> top_vertices[insertpos].id = 
      static_cast<char*>(malloc1(sizeof(char) * (l+1)));
    name.copy(geo_ -> top_vertices[insertpos].id, l);
    geo_ -> top_vertices[insertpos].id[l] = 0;
    ++(geo_ -> num_top_vertices);
    return Brep::Face_Spec(0,insertpos);
  }
  else if (dim_ == 1) {
    int insertpos = geo_ -> num_top_edges;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      Cptc_TEdge* new_edges = 
        static_cast<Cptc_TEdge*>(malloc1(sizeof(Cptc_TEdge) * newsize));
      for (int j = 0; j < insertpos; ++j) {
        new_edges[j] = geo_ -> top_edges[j];
      }
      free(geo_ -> top_edges);
      maxsize_ = newsize;
      geo_ -> top_edges = new_edges;
    }      
    geo_ -> top_edges[insertpos].props = 0;
    geo_ -> top_edges[insertpos].values = 0;
    geo_ -> top_edges[insertpos].bound_vertices = 0;
    geo_ -> top_edges[insertpos].num_bound_vertices = 0;
    geo_ -> top_edges[insertpos].num_curves = 0;
    geo_ -> top_edges[insertpos].curves = 0;
    geo_ -> top_edges[insertpos].num_properties = 0;
    int l = name.length();
    geo_ -> top_edges[insertpos].id = 
      static_cast<char*>(malloc1(sizeof(char) * (l+1)));
    name.copy(geo_ -> top_edges[insertpos].id, l);
    geo_ -> top_edges[insertpos].id[l] = 0;
    ++(geo_ -> num_top_edges);
    return Brep::Face_Spec(1,insertpos);
  }
  else if (dim_ == 2) {
    int insertpos = geo_ -> num_top_surfaces;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      Cptc_TSurface* new_surfaces = 
        static_cast<Cptc_TSurface*>(malloc1(sizeof(Cptc_TSurface) * newsize));
      for (int j = 0; j < insertpos; ++j) {
        new_surfaces[j] = geo_ -> top_surfaces[j];
      }
      free(geo_ -> top_surfaces);
      maxsize_ = newsize;
      geo_ -> top_surfaces = new_surfaces;
    }      
    geo_ -> top_surfaces[insertpos].props = 0;
    geo_ -> top_surfaces[insertpos].values = 0;
    geo_ -> top_surfaces[insertpos].bound_vertices = 0;
    geo_ -> top_surfaces[insertpos].num_bound_vertices = 0;
    geo_ -> top_surfaces[insertpos].bound_edges = 0;
    geo_ -> top_surfaces[insertpos].num_bound_edges = 0;
    geo_ -> top_surfaces[insertpos].num_patches = 0;
    geo_ -> top_surfaces[insertpos].patches = 0;
    geo_ -> top_surfaces[insertpos].num_properties = 0;
    int l = name.length();
    geo_ -> top_surfaces[insertpos].id = 
      static_cast<char*>(malloc1(sizeof(char) * (l+1)));
    name.copy(geo_ -> top_surfaces[insertpos].id, l);
    geo_ -> top_surfaces[insertpos].id[l] = 0;
    ++(geo_ -> num_top_surfaces);
    return Brep::Face_Spec(2,insertpos);
  }
  // else if dim_ == 3
  int insertpos = geo_ -> num_top_regions;
  if (insertpos == maxsize_) {
    int newsize = maxsize_ * 2 + 1;
    Cptc_TRegion* new_regions = 
      static_cast<Cptc_TRegion*>(malloc1(sizeof(Cptc_TRegion) * newsize));
    for (int j = 0; j < insertpos; ++j) {
      new_regions[j] = geo_ -> top_regions[j];
    }
    free(geo_ -> top_regions);
    maxsize_ = newsize;
    geo_ -> top_regions = new_regions;
  }      
  geo_ -> top_regions[insertpos].props = 0;
  geo_ -> top_regions[insertpos].values = 0;
  geo_ -> top_regions[insertpos].bound_vertices = 0;
  geo_ -> top_regions[insertpos].num_bound_vertices = 0;
  geo_ -> top_regions[insertpos].bound_edges = 0;
  geo_ -> top_regions[insertpos].num_bound_edges = 0;
  geo_ -> top_regions[insertpos].num_bound_surfaces = 0;
  geo_ -> top_regions[insertpos].bound_surfaces = 0;
  geo_ -> top_regions[insertpos].surface_orientation = 0;
  geo_ -> top_regions[insertpos].num_properties = 0;
  int l = name.length();
  geo_ -> top_regions[insertpos].id = 
    static_cast<char*>(malloc1(sizeof(char) * (l+1)));
  name.copy(geo_ -> top_regions[insertpos].id, l);
  geo_ -> top_regions[insertpos].id[l] = 0;
  ++(geo_ -> num_top_regions);
  return Brep::Face_Spec(3,insertpos);
}



QMG::Brep_Under_Construction::Top_face_bound_inserter::
Top_face_bound_inserter(Brep_Under_Construction& brep, 
                        const Brep::Face_Spec& fspec) : brep_(brep) {
  geo_ = brep.geo_;
  fspec_ = fspec;
  maxsize_ = 1;
  int ind = fspec_.faceind();
  int dim = fspec_.fdim();
  if (dim < 0 || dim > geo_ -> intrinsic_dimension)
    throw_error("Dimension out of range");
  if (fspec_.faceind() < 0) 
    throw_error("Faceindex out of range");
  if (dim == 0) {
    if (ind >= geo_ -> num_top_vertices)
      throw_error("Faceindex out of range (0)");
  }
  else if (dim == 1) {
    if (ind >= geo_-> num_top_edges)
      throw_error("Faceindex out of range (1)");
    if (geo_ -> top_edges[ind].bound_vertices != 0)
      throw_error("Illegal to create top edge bound face inserter more than once");
    geo_-> top_edges[ind].bound_vertices = 
      static_cast<int*>(malloc1(sizeof(int)));
    geo_ -> top_edges[ind].num_bound_vertices = 0;
  }
  else if (dim == 2) {
    if (ind >= geo_-> num_top_surfaces)
      throw_error("Faceindex out of range (2)");
    if (geo_ -> top_surfaces[ind].bound_edges != 0)
      throw_error("Illegal to create top edge bound surface inserter more than once");
    geo_-> top_surfaces[ind].bound_edges = 
      static_cast<int*>(malloc1(sizeof(int)));
    geo_ -> top_surfaces[ind].num_bound_edges = 0;
  }
  else  { // if dim == 3 
     if (ind >= geo_-> num_top_regions)
      throw_error("Faceindex out of range (3)");
    if (geo_ -> top_regions[ind].bound_surfaces != 0)
      throw_error("Illegal to create top edge bound region inserter more than once");
    geo_-> top_regions[ind].bound_surfaces = 
      static_cast<int*>(malloc1(sizeof(int)));
    geo_-> top_regions[ind].surface_orientation = 
      static_cast<int*>(malloc1(sizeof(int)));
    geo_ -> top_regions[ind].num_bound_surfaces = 0;
  }
}



void
QMG::Brep_Under_Construction::Top_face_bound_inserter::
insert_boundary(const Brep::Face_Spec& subfspec, 
                int orientation) { 
  int dim = fspec_.fdim();
  if (subfspec.fdim() != dim - 1) {
    throw_error("Attempt to insert bounding face of wrong dimension.");
  }
  int subind = subfspec.faceind();
  int ind = fspec_.faceind();
  if (subind < 0) 
    throw_error("Subind out of range (0)");
  if (dim == 0) {
    throw_error("Illegal to insert bounding face into vertex");
  }
  else if (dim == 1) {
    if (subind >= geo_ -> num_top_vertices) {
      throw_error("Subind out of range (1)");
    }
    int insertpos = geo_ -> top_edges[ind].num_bound_vertices;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      int* new_bound = 
        static_cast<int*>(malloc1(sizeof(int) * newsize));
      for (int j = 0; j < insertpos; ++j) {
        new_bound[j] = geo_ -> top_edges[ind].bound_vertices[j];
      }
      free(geo_ -> top_edges[ind].bound_vertices);
      maxsize_ = newsize;
      geo_ -> top_edges[ind].bound_vertices = new_bound;
    }      
    geo_ -> top_edges[ind].bound_vertices[insertpos] = subind;
    ++(geo_ -> top_edges[ind].num_bound_vertices);
  }
  else if (dim == 2) {
    if (subind >= geo_ -> num_top_edges) {
      throw_error("Subind out of range (2)");
    }
    int insertpos = geo_ -> top_surfaces[ind].num_bound_edges;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      int* new_bound = 
        static_cast<int*>(malloc1(sizeof(int) * newsize));
      for (int j = 0; j < insertpos; ++j) {
        new_bound[j] = geo_ -> top_surfaces[ind].bound_edges[j];
      }
      free(geo_ -> top_surfaces[ind].bound_edges);
      maxsize_ = newsize;
      geo_ -> top_surfaces[ind].bound_edges = new_bound;
    }      
    geo_ -> top_surfaces[ind].bound_edges[insertpos] = subind;
    ++(geo_ -> top_surfaces[ind].num_bound_edges);
  }
  else if (dim == 3) {
    if (subind >= geo_ -> num_top_surfaces) {
      throw_error("Subind out of range (3)");
    }
    int insertpos = geo_ -> top_regions[ind].num_bound_surfaces;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      int* new_bound = 
        static_cast<int*>(malloc1(sizeof(int) * newsize));
      {
        for (int j = 0; j < insertpos; ++j) {
          new_bound[j] = geo_ -> top_regions[ind].bound_surfaces[j];
        }
      }
      free(geo_ -> top_regions[ind].bound_surfaces);
      maxsize_ = newsize;
      geo_ -> top_regions[ind].bound_surfaces = new_bound;
      int* new_ori = 
        static_cast<int*>(malloc1(sizeof(int) * newsize));
      {
        for (int j = 0; j < insertpos; ++j)
          new_ori[j] = geo_ -> top_regions[ind].surface_orientation[j];
      }
      free(geo_ -> top_regions[ind].surface_orientation);
      geo_ -> top_regions[ind].surface_orientation = new_ori;
    }      
    geo_ -> top_regions[ind].bound_surfaces[insertpos] = subind;
    geo_ -> top_regions[ind].surface_orientation[insertpos] = orientation;
    ++(geo_ -> top_regions[ind].num_bound_surfaces);
  }
}



QMG::Brep_Under_Construction::Top_face_ib_inserter::
Top_face_ib_inserter(Brep_Under_Construction& brep,
                     const Brep::Face_Spec& fspec) : brep_(brep) {
  geo_ = brep.geo_;
  fspec_ = fspec;
  maxsize1_ = 1;
  int ind = fspec_.faceind();
  int dim = fspec_.fdim();
  if (dim < 0 || dim > geo_ -> intrinsic_dimension)
    throw_error("Dimension out of range");
  if (fspec_.faceind() < 0) 
    throw_error("Faceindex out of range");
  if (dim == 0) {
    if (ind >= geo_ -> num_top_vertices)
      throw_error("Faceindex out of range (3a)");
  }
  else if (dim == 1) {
    if (ind >= geo_ -> num_top_edges)
      throw_error("Faceindex out of range (3b)");
  }
  else if (dim == 2) {
    if (ind >= geo_-> num_top_surfaces)
      throw_error("Faceindex out of range (4)");
    if (geo_ -> top_surfaces[ind].bound_vertices != 0)
      throw_error("Illegal to create top surface ib inserter more than once");
    geo_-> top_surfaces[ind].bound_vertices = 
      static_cast<int*>(malloc1(sizeof(int)));
    geo_ -> top_surfaces[ind].num_bound_vertices = 0;
  }
  else  { // if dim == 3 
     if (ind >= geo_-> num_top_regions)
      throw_error("Faceindex out of range (5)");
    if (geo_ -> top_regions[ind].bound_edges != 0 ||
      geo_ -> top_regions[ind].bound_vertices != 0)
      throw_error("Illegal to create top region ib inserter more than once");
    geo_-> top_regions[ind].bound_vertices = 
      static_cast<int*>(malloc1(sizeof(int)));
    maxsize2_ = 0;
    geo_-> top_regions[ind].bound_edges = 
      static_cast<int*>(malloc1(sizeof(int)));
    geo_ -> top_regions[ind].num_bound_edges = 0;
    geo_ -> top_regions[ind].num_bound_vertices = 0;
  }
}



void
QMG::Brep_Under_Construction::Top_face_ib_inserter::
insert_ib(const Brep::Face_Spec& subfspec) {
  int dim = fspec_.fdim();
  int subdim = subfspec.fdim();
  if (subdim == dim - 1) {
    throw_error("Insert ib of one lower dimension by including twice in bound face list.");
  }
  if (subdim >= dim || subdim < 0) {
    throw_error("Illegal dimension of subface to insert ib");
  }
  int subind = subfspec.faceind();
  int ind = fspec_.faceind();
  if (subind < 0) 
    throw_error("Subind out of range (0)");
  if (dim == 2) {
    if (subind >= geo_ -> num_top_vertices) {
      throw_error("Subind out of range (4)");
    }
    int insertpos = geo_ -> top_surfaces[ind].num_bound_vertices;
    if (insertpos == maxsize1_) {
      int newsize = maxsize1_ * 2 + 1;
      int* new_bound = 
        static_cast<int*>(malloc1(sizeof(int) * newsize));
      for (int j = 0; j < insertpos; ++j) {
        new_bound[j] = geo_ -> top_surfaces[ind].bound_vertices[j];
      }
      free(geo_ -> top_surfaces[ind].bound_vertices);
      maxsize1_ = newsize;
      geo_ -> top_surfaces[ind].bound_vertices = new_bound;
    }      
    geo_ -> top_surfaces[ind].bound_vertices[insertpos] = subind;
    ++(geo_ -> top_surfaces[ind].num_bound_vertices);
  }
  else { //if dim == 3
    if (subdim == 0) {
      if (subind >= geo_ -> num_top_vertices) {
        throw_error("Subind out of range (5)");
      }
      int insertpos = geo_ -> top_regions[ind].num_bound_vertices;
      if (insertpos == maxsize1_) {
        int newsize = maxsize1_ * 2 + 1;
        int* new_bound = 
          static_cast<int*>(malloc1(sizeof(int) * newsize));
        {
          for (int j = 0; j < insertpos; ++j) {
            new_bound[j] = geo_ -> top_regions[ind].bound_vertices[j];
          }
        }
        free(geo_ -> top_regions[ind].bound_vertices);
        maxsize1_ = newsize;
        geo_ -> top_regions[ind].bound_vertices = new_bound;
      }      
      geo_ -> top_regions[ind].bound_vertices[insertpos] = subind;
      ++(geo_ -> top_regions[ind].num_bound_vertices);
    }
    else { //subdim == 1
      if (subind >= geo_ -> num_top_edges) {
        throw_error("Subind out of range (5)");
      }
      int insertpos = geo_ -> top_regions[ind].num_bound_edges;
      if (insertpos == maxsize2_) {
        int newsize = maxsize2_ * 2 + 1;
        int* new_bound = 
          static_cast<int*>(malloc1(sizeof(int) * newsize));
        {
          for (int j = 0; j < insertpos; ++j) {
            new_bound[j] = geo_ -> top_regions[ind].bound_edges[j];
          }
        }
        free(geo_ -> top_regions[ind].bound_edges);;
        maxsize2_ = newsize;
        geo_ -> top_regions[ind].bound_edges = new_bound;
      }      
      geo_ -> top_regions[ind].bound_edges[insertpos] = subind;
      ++(geo_ -> top_regions[ind].num_bound_edges);
    }
  }
}



QMG::Brep_Under_Construction::Top_face_patch_inserter::
Top_face_patch_inserter(Brep_Under_Construction& brep,
                        const Brep::Face_Spec& fspec) : brep_(brep) {
  geo_ = brep.geo_;
  fspec_ = fspec;
  maxsize_ = 1;
  int ind = fspec_.faceind();
  int dim = fspec_.fdim();
  if (dim < 0 || dim > geo_ -> intrinsic_dimension)
    throw_error("Dimension out of range");
  if (fspec_.faceind() < 0) 
    throw_error("Faceindex out of range");
  if (dim == 0) {
    if (ind >= geo_ -> num_top_vertices) 
      throw_error("Faceindex out of range (7)");
    if (geo_ -> top_vertices[ind].control_pnt_indx >= 0) {
      throw_error("Illegal to create top vertex patch inserter more than once");
    }
  }
  else if (dim == 1) {
    if (ind >= geo_ -> num_top_edges) 
      throw_error("Faceindex out of range (8)");
    if (geo_ -> top_edges[ind].curves != 0)
      throw_error("Illegal to create top edge patch inserter more than once");
    geo_ -> top_edges[ind].curves =
      static_cast<Cptc_GCurve*>(malloc1(sizeof(Cptc_GCurve)));
    geo_ -> top_edges[ind].num_curves = 0;
  }
  else if (dim == 2) {
    if (ind >= geo_ -> num_top_surfaces)
      throw_error("Faceindex out of range (9)");
    if (geo_ -> top_surfaces[ind].patches != 0)
      throw_error("Illegal to create top surface patch inserter more than once");
    geo_-> top_surfaces[ind].patches = 
      static_cast<Cptc_GPatch*>(malloc1(sizeof(Cptc_GPatch)));
    geo_ -> top_surfaces[ind].num_patches = 0;
  }
  else if (dim == 3) {
    if (ind >= geo_ -> num_top_regions)
      throw_error("Faceindex out of range (9a)");
  }
}


void 
QMG::Brep_Under_Construction::Top_face_patch_inserter::
insert_patch(int degree1, 
             int degree2,
             PatchType ptype,
             const vector<Brep::ControlPointIndex>&  control_points) {
  int dim = fspec_.fdim();
  int ind = fspec_.faceind();

  if (dim == geo_ -> embedded_dimension) {
    throw_error("Illegal to insert patches into chamber");
  }
  for (int jj = 0; jj < control_points.size(); ++jj) {
    if (control_points[jj] < 0 || control_points[jj] >= geo_ -> num_control_pts) {
      throw_error("Control point index out of range");
    }
  }
  if (dim == 0) {
    if (geo_ -> top_vertices[ind].control_pnt_indx >= 0) 
      throw_error("Illegal to have more than one control point for topological vertex");
    if (control_points.size() != 1) {
      throw_error("Topological vertex takes exactly one control point");
    }
    geo_ -> top_vertices[ind].control_pnt_indx = control_points[0];
  }  
  else if (dim == 1) {
    int numcp = degree1 + 1;
    if (control_points.size() != numcp) {
      throw_error("Incorrect number of control points in curve");
    }
    int insertpos = geo_ -> top_edges[ind].num_curves;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      Cptc_GCurve* new_curves = 
        static_cast<Cptc_GCurve*>(malloc1(sizeof(Cptc_GCurve) * newsize));
      for (int j = 0; j < insertpos; ++j) {
        new_curves[j] = geo_ -> top_edges[ind].curves[j];
      }
      free(geo_ -> top_edges[ind].curves);
      maxsize_ = newsize;
      geo_ -> top_edges[ind].curves = new_curves;
    }      
    geo_ -> top_edges[ind].curves[insertpos].order = degree1;
    geo_ -> top_edges[ind].curves[insertpos].control_pnt_indx =
      static_cast<int*>(malloc1(sizeof(int) * numcp));
    for (int j = 0; j < numcp; ++j) {
      geo_ -> top_edges[ind].curves[insertpos].control_pnt_indx[j] =
        control_points[j];
    }
    ++(geo_ -> top_edges[ind].num_curves);
  }
  else { // if dim == 2
    int numcp = (ptype == BEZIER_QUAD)? (degree1 + 1) * (degree2 + 1) :
      ((degree1 + 1) * (degree1 + 2) / 2);
    if (control_points.size() != numcp) {
      ostringstream errstr;
      errstr << "Incorrect number of control points in patch\n";
      errstr << "For patch of type ";
      if (ptype == BEZIER_QUAD) 
        errstr << "QUAD degree (" << degree1 << ", " << degree2 << ")";
      else
        errstr << "TRIANGLE degree "<< degree1;      
      errstr << " expected number is " << numcp << " but actual number read is "
        << control_points.size();
      throw_error(errstr.str());
    }
    int insertpos = geo_ -> top_surfaces[ind].num_patches;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      Cptc_GPatch* new_patches = 
        static_cast<Cptc_GPatch*>(malloc1(sizeof(Cptc_GPatch) * newsize));
      for (int j = 0; j < insertpos; ++j) {
        new_patches[j] = geo_ -> top_surfaces[ind].patches[j];
      }
      free(geo_ -> top_surfaces[ind].patches);
      maxsize_ = newsize;
      geo_ -> top_surfaces[ind].patches = new_patches;
    }      
    geo_ -> top_surfaces[ind].patches[insertpos].order_u = degree1;
    if (ptype == BEZIER_QUAD)
      geo_ -> top_surfaces[ind].patches[insertpos].order_v = degree2;
    geo_ -> top_surfaces[ind].patches[insertpos].type = 
      (ptype == BEZIER_QUAD) ? CPTC_PATCHTYPE_QUAD : CPTC_PATCHTYPE_TRIANGLE;
    geo_ -> top_surfaces[ind].patches[insertpos].control_pnt_indx =
      static_cast<int*>(malloc1(sizeof(int) * numcp));
    for (int j = 0; j < numcp; ++j) {
      geo_ -> top_surfaces[ind].patches[insertpos].control_pnt_indx[j] =
        control_points[j];
    }
    ++(geo_ -> top_surfaces[ind].num_patches);
  }
}


QMG::Brep_Under_Construction::Top_face_propval_inserter::
Top_face_propval_inserter(Brep_Under_Construction& brep, 
                          const Brep::Face_Spec& fspec) : brep_(brep) {
  geo_ = brep.geo_;
  fspec_ = fspec;
  maxsize_ = 1;
  int ind = fspec_.faceind();
  int dim = fspec_.fdim();
  if (dim < 0 || dim > geo_ -> intrinsic_dimension)
    throw_error("Dimension out of range");
  if (fspec_.faceind() < 0) 
    throw_error("Faceindex out of range");
  if (dim == 0) {
    if (ind >= geo_ -> num_top_vertices)
      throw_error("Faceindex out of range (10)");
    if (geo_ -> top_vertices[ind].props != 0)
      throw_error("Illegal to create top vertex prop-val inserter more than once");
    geo_ -> top_vertices[ind].props =
      static_cast<char**>(malloc1(sizeof(char*)));
    geo_ -> top_vertices[ind].values =
      static_cast<char**>(malloc1(sizeof(char*)));
    geo_ -> top_vertices[ind].num_properties = 0;
  }
  else if (dim == 1) {
    if (ind >= geo_ -> num_top_edges)
      throw_error("Faceindex out of range (11)");
    if (geo_ -> top_edges[ind].props != 0)
      throw_error("Illegal to create top edge prop-val inserter more than once");
    geo_ -> top_edges[ind].props =
      static_cast<char**>(malloc1(sizeof(char*)));
    geo_ -> top_edges[ind].values =
      static_cast<char**>(malloc1(sizeof(char*)));
    geo_ -> top_edges[ind].num_properties = 0;
  }
  else if (dim == 2) {
    if (ind >= geo_ -> num_top_surfaces)
      throw_error("Faceindex out of range (12)");
    if (geo_ -> top_surfaces[ind].props != 0)
      throw_error("Illegal to create top surface prop-val inserter more than once");
    geo_ -> top_surfaces[ind].props =
      static_cast<char**>(malloc1(sizeof(char*)));
    geo_ -> top_surfaces[ind].values =
      static_cast<char**>(malloc1(sizeof(char*)));
    geo_ -> top_surfaces[ind].num_properties = 0;
  }
  else { // if dim==3
    if (ind >= geo_ -> num_top_regions)
      throw_error("Faceindex out of range (13)");
    if (geo_ -> top_regions[ind].props != 0)
      throw_error("Illegal to create top surface prop-val inserter more than once");
    geo_ -> top_regions[ind].props =
      static_cast<char**>(malloc1(sizeof(char*)));
    geo_ -> top_regions[ind].values =
      static_cast<char**>(malloc1(sizeof(char*)));
    geo_ -> top_regions[ind].num_properties = 0;
  }
}


void
QMG::Brep_Under_Construction::Top_face_propval_inserter::
insert_propval(const string& prop, const string& val) {
  int dim = fspec_.fdim();
  int ind = fspec_.faceind();
  if (dim == 0) {
    int insertpos = geo_ -> top_vertices[ind].num_properties;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      char** new_prop = 
        static_cast<char**>(malloc1(sizeof(char*) * newsize));
      char** new_val = 
        static_cast<char**>(malloc1(sizeof(char*) * newsize));
      {
        for (int j = 0; j < insertpos; ++j) {
          new_prop[j] = geo_ -> top_vertices[ind].props[j];
          new_val[j] = geo_ -> top_vertices[ind].values[j];
        }
      }
      free(geo_ -> top_vertices[ind].props);
      free(geo_ -> top_vertices[ind].values);
      geo_ -> top_vertices[ind].props = new_prop;
      geo_ -> top_vertices[ind].values = new_val;
      maxsize_ = newsize;
    }
    int l1 = prop.length();
    geo_ -> top_vertices[ind].props[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l1 + 1)));
    prop.copy(geo_ -> top_vertices[ind].props[insertpos], l1);
    geo_ -> top_vertices[ind].props[insertpos][l1] = 0;
    int l2 = val.length();
    geo_ -> top_vertices[ind].values[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l2 + 1)));
    val.copy(geo_ -> top_vertices[ind].values[insertpos], l2);
    geo_ -> top_vertices[ind].values[insertpos][l2] = 0;
    ++(geo_ -> top_vertices[ind].num_properties);
   // std::cerr << " add property " << insertpos << " to vertex " << ind << " prop = "
    //  << geo_ -> top_vertices[ind].props[insertpos] <<  " value = "
     // << geo_ -> top_vertices[ind].values[insertpos] << "\n";
    // dump_vtx_prop_val(geo_,ind);
  }
  else if (dim == 1) {
    int insertpos = geo_ -> top_edges[ind].num_properties;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      char** new_prop = 
        static_cast<char**>(malloc1(sizeof(char*) * newsize));
      char** new_val = 
        static_cast<char**>(malloc1(sizeof(char*) * newsize));
      {
        for (int j = 0; j < insertpos; ++j) {
          new_prop[j] = geo_ -> top_edges[ind].props[j];
          new_val[j] = geo_ -> top_edges[ind].values[j];
        }
      }
      free(geo_ -> top_edges[ind].props);
      free(geo_ -> top_edges[ind].values);
      geo_ -> top_edges[ind].props = new_prop;
      geo_ -> top_edges[ind].values = new_val;
      maxsize_ = newsize;
    }
    int l1 = prop.length();
    geo_ -> top_edges[ind].props[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l1 + 1)));
    prop.copy(geo_ -> top_edges[ind].props[insertpos], l1);
    geo_ -> top_edges[ind].props[insertpos][l1] = 0;
    int l2 = val.length();
    geo_ -> top_edges[ind].values[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l2 + 1)));
    val.copy(geo_ -> top_edges[ind].values[insertpos], l2);
    geo_ -> top_edges[ind].values[insertpos][l2] = 0;
    ++(geo_ -> top_edges[ind].num_properties);
  }
  else if (dim == 2) {
    int insertpos = geo_ -> top_surfaces[ind].num_properties;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      char** new_prop = 
        static_cast<char**>(malloc1(sizeof(char*) * newsize));
      char** new_val = 
        static_cast<char**>(malloc1(sizeof(char*) * newsize));
      {
        for (int j = 0; j < insertpos; ++j) {
          new_prop[j] = geo_ -> top_surfaces[ind].props[j];
          new_val[j] = geo_ -> top_surfaces[ind].values[j];
        }
      }
      free(geo_ -> top_surfaces[ind].props);
      free(geo_ -> top_surfaces[ind].values);
      geo_ -> top_surfaces[ind].props = new_prop;
      geo_ -> top_surfaces[ind].values = new_val;
      maxsize_ = newsize;
    }
    int l1 = prop.length();
    geo_ -> top_surfaces[ind].props[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l1 + 1)));
    prop.copy(geo_ -> top_surfaces[ind].props[insertpos], l1);
    geo_ -> top_surfaces[ind].props[insertpos][l1] = 0;
    int l2 = val.length();
    geo_ -> top_surfaces[ind].values[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l2 + 1)));
    val.copy(geo_ -> top_surfaces[ind].values[insertpos], l2);
    geo_ -> top_surfaces[ind].values[insertpos][l2] = 0;
    ++(geo_ -> top_surfaces[ind].num_properties);
  }
  else { // if dim == 3
    int insertpos = geo_ -> top_regions[ind].num_properties;
    if (insertpos == maxsize_) {
      int newsize = maxsize_ * 2 + 1;
      char** new_prop = 
        static_cast<char**>(malloc1(sizeof(char*) * newsize));
      char** new_val = 
        static_cast<char**>(malloc1(sizeof(char*) * newsize));
      {
        for (int j = 0; j < insertpos; ++j) {
          new_prop[j] = geo_ -> top_regions[ind].props[j];
          new_val[j] = geo_ -> top_regions[ind].values[j];
        }
      }
      free(geo_ -> top_regions[ind].props);
      free(geo_ -> top_regions[ind].values);
      geo_ -> top_regions[ind].props = new_prop;
      geo_ -> top_regions[ind].values = new_val;
      maxsize_ = newsize;
    }
    int l1 = prop.length();
    geo_ -> top_regions[ind].props[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l1 + 1)));
    prop.copy(geo_ -> top_regions[ind].props[insertpos], l1);
    geo_ -> top_regions[ind].props[insertpos][l1] = 0;
    int l2 = val.length();
    geo_ -> top_regions[ind].values[insertpos] = static_cast<char*>(
      malloc1(sizeof(char) * (l2 + 1)));
    val.copy(geo_ -> top_regions[ind].values[insertpos], l2);
    geo_ -> top_regions[ind].values[insertpos][l2] = 0;
    ++(geo_ -> top_regions[ind].num_properties);
  }
}


